/*
 *
 *   (C) Copyright IBM Corp. 2003
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or 
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software 
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
                   
extern plugin_record_t ogfs_plugin_record;
extern plugin_record_t *my_plugin_record;
extern engine_functions_t *EngFncs;
                 
#define FS_TYPE_OGFS			20
#define OGFS_PLUGIN_INFO_COUNT		6
#define OGFS_CG_INFO_COUNT		5
#define OGFS_SB_INFO_COUNT		4

#define OGFS_SB_OFFSET			(OGFS_SB_ADDR * OGFS_BASIC_BLOCK)
#define OGFS_LH_OFFSET			0
#define OGFS_CG_OFFSET			0

#define MINOGFS				(0x1000000)

/* Types of OGFS volumes */
typedef enum {
	OGFS_FS_VOL,
	OGFS_CI_VOL,
	OGFS_JOURNAL_VOL,
	OGFS_UNKNOWN_VOL
} ogfs_volume_type_t;

/*
 * Hopefully temporary linked list routines. The list is doubly-linked and circular.
 */

struct list_element {
	struct list_element *next;
	struct list_element *prev;
	void *data;
};

static inline struct list_element *append_list_element(struct list_element **head, void *data)
{
	struct list_element *new;
	
	new = malloc(sizeof(struct list_element));

	if (new != NULL) {
		if (*head != NULL) {
			/* head->prev always points to tail so append new element */
			/* and adjust other's next and prev pointers accordingly */
			new->prev = (*head)->prev;
			new->prev->next = new;
			(*head)->prev = new;
		} else {
			/* first item on list. everything points to it! */
			*head = new;
			new->prev = new;
		}
		new->next = *head;
		new->data = data;
	}
	return new;	
}

static inline void delete_list_element(struct list_element **head, struct list_element *element)
{
	if (element->next == element->prev) {
		*head = NULL;
	} else {
		if (*head == element)
			*head = element->next;
		element->next->prev = element->prev;
		element->prev->next = element->next;
	}
	free(element);
}

static inline void delete_all_list_elements(struct list_element *head, boolean free_data)
{
	if (head != NULL) {
		struct list_element *tmp = NULL, *curr;
	
		curr = head;
		do {
			if (free_data == TRUE) {
				if (curr->data != NULL)
					free(curr->data);
			}
			tmp = curr;
			curr = curr->next;
			free(tmp);		
		} while (curr != head );
	}
}

static inline struct list_element *duplicate_list(struct list_element *head)
{
	struct list_element *new = NULL;
	
	if (head != NULL) {
		struct list_element *curr;
		
		curr = head;
		do {
			append_list_element(&new, curr->data);
			curr = curr->next;
		} while (curr != head);
	}
	return new;
}
