/*
 *   (C) Copyright IBM Corp. 2001, 2003
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * get_dev_num
 *
 * Simple command-line utility to print the device number for a specified
 * block device. The device number is printed as a single, decimal number.
 *    number = major * 256 + minor
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>

int main(int argc, char *argv[])
{
	struct stat file_info;
	int rc;

	if (argc != 2) {
		fprintf(stderr, "USAGE: %s device_file\n", argv[0]);
		return 1;
	}

	rc = stat(argv[1], &file_info);
	if (rc) {
		fprintf(stderr, "Error returned from stat(): %s\n",
			strerror(errno));
		return 1;
	}

	if (!S_ISBLK(file_info.st_mode)) {
		fprintf(stderr, "%s is not a valid block device\n", argv[1]);
		return 1;
	}

	printf("%d\n", (int)file_info.st_rdev);

	return 0;
}

